
class FundamentantalAccountingConcepts:               

    def __init__(self, xbrl):               

        self.xbrl = xbrl        

        #self.xbrl.fields['Assets']
        
        #TODO:parse fixtures/normalized_fields.csv
